'use strict';

var libQ = require('kew');
var fs = require('fs-extra');
var Gpio = require('onoff').Gpio;
var io = require('socket.io-client');
var socket = io.connect('http://localhost:3000');
var actions = ["playPause", "volumeUp", "volumeDown", "previous", "next", "shutdown", "toggleMute", "toggleRandom", "toggleRepeat", "toggleRepeatAll", "toggleRepeatSingle"];

module.exports = GPIOButtons;

function GPIOButtons(context) {
	var self = this;
	self.context=context;
	self.commandRouter = self.context.coreCommand;
	self.logger = self.context.logger;
	self.triggers = [];
}


GPIOButtons.prototype.onVolumioStart = function () {
	var self = this;

	var configFile=this.commandRouter.pluginManager.getConfigurationFile(this.context,'config.json');
	this.config = new (require('v-conf'))();
	this.config.loadFile(configFile);

	self.logger.info("GPIO-Buttons initialized");
	
	return libQ.resolve();	
};


GPIOButtons.prototype.getConfigurationFiles = function()
{
	return ['config.json'];
};


GPIOButtons.prototype.onStart = function () {
	var self = this;
	var defer=libQ.defer();

	self.createTriggers()
		.then (function (result) {
			self.logger.info("GPIO-Buttons started");
			defer.resolve();
		});
	
    return defer.promise;
};


GPIOButtons.prototype.onStop = function () {
	var self = this;
	var defer=libQ.defer();

	self.clearTriggers()
		.then (function (result) {
			self.logger.info("GPIO-Buttons stopped");
			defer.resolve();
		});
	
    return defer.promise;
};


GPIOButtons.prototype.onRestart = function () {
	var self = this;
};

GPIOButtons.prototype.onInstall = function () {
	var self = this;
};

GPIOButtons.prototype.onUninstall = function () {
	var self = this;
};

GPIOButtons.prototype.getConf = function (varName) {
	var self = this;
};

GPIOButtons.prototype.setConf = function(varName, varValue) {
	var self = this;
};

GPIOButtons.prototype.getAdditionalConf = function (type, controller, data) {
	var self = this;
};

GPIOButtons.prototype.setAdditionalConf = function () {
	var self = this;
};

GPIOButtons.prototype.setUIConfig = function (data) {
	var self = this;
};


GPIOButtons.prototype.getUIConfig = function () {
	var defer = libQ.defer();
	var self = this;

	self.logger.info('GPIO-Buttons: Getting UI config');

	//Just for now..
	var lang_code = 'en';

	//var lang_code = this.commandRouter.sharedVars.get('language_code');

        self.commandRouter.i18nJson(__dirname+'/i18n/strings_'+lang_code+'.json',
                __dirname+'/i18n/strings_en.json',
                __dirname + '/UIConfig.json')
        .then(function(uiconf)
        {
			actions.forEach(function(action, index, array) {
 				
                // strings for UI data
                var s1 = action.concat("Enabled");
				var s2 = action.concat("Pin");
                
 				// Strings for config
				var c1 = action.concat('.enabled');
				var c2 = action.concat('.pin');
				
                // Extend the find method on the content array
				uiconf.sections[0].content.findItem = function(obj) {
					return this.find(function(item) {
						for (var prop in obj)
							if (!(prop in item) || obj[prop] !== item[prop])
								 return false;
						return true;
					});
				}
                
                self.setUiValue(uiconf, s1, self.config.get(c1), false);
                self.setUiValueAndLabel(uiconf, s2, self.config.get(c2), 0);
			});

            defer.resolve(uiconf);
		})
        .fail(function()
        {
            defer.reject(new Error());
        });

        return defer.promise;
};


GPIOButtons.prototype.saveConfig = function(data)
{
	var self = this;

	actions.forEach(function(action, index, array) {
 		// Strings for data fields
		var s1 = action.concat('Enabled');
		var s2 = action.concat('Pin');

		// Strings for config
		var c1 = action.concat('.enabled');
		var c2 = action.concat('.pin');
		var c3 = action.concat('.value');
        
		self.setConfigValue(c1, "boolean", data[s1]);
		self.setConfigValue(c2, "number", data[s2]['value']);
		self.setConfigValue(c3, "number", 0);
    });

    //self.logger.info('Saved config');
    //self.logger.info(self.config.print());
        
	self.clearTriggers()
		.then(self.createTriggers());

	self.commandRouter.pushToastMessage('success',"GPIO-Buttons", "Configuration saved");
};

// Set a config value, or create it if it doesn't exist.
// According to v-conf docs, config.set is supposed to create an entry
// if it doesn't exist, but this doesn't appear to happen.
GPIOButtons.prototype.setConfigValue = function(key, typeName, value) {
    var self = this;
    
    //self.logger.info("Setting config key " + key + " to value " + value);

    if(self.config.has(key))
        self.config.set(key, value);
    else
        self.config.addConfigValue(key, typeName, value);
}

// Set value for a UI element
GPIOButtons.prototype.setUiValue = function(uiconf, field, value, defaultValue){
	var self = this;
	var result = self.getUIElement(uiconf, field);
	if (result)
    {
        if (value === undefined) {
            // value is not present in config - perhaps the user's config is from an older version.
            result.value = defaultValue;
        }
        else {
            result.value = value;
        }
    }
}


// Set value and label for a UI element
GPIOButtons.prototype.setUiValueAndLabel = function(uiconf, field, value, defaultValue){
	var self = this;
	var result = self.getUIElement(uiconf, field);
	if (result){
        if(value === undefined) {
            // value is not present in config - perhaps the user's config is from an older version.
            result.value.value = defaultValue;
            result.value.label = defaultValue.toString();
        }
        else {
            result.value.value = value;
            result.value.label = value.toString();
        }
	}
}


// Retrieve a UI element from UI config
GPIOButtons.prototype.getUIElement = function(obj, field){
	var self = this;
	var lookfor = JSON.parse('{"id":"' + field + '"}');
	return obj.sections[0].content.findItem(lookfor);
}


GPIOButtons.prototype.createTriggers = function() {
	var self = this;

	self.logger.info('GPIO-Buttons: Reading config and creating triggers...');

	actions.forEach(function(action, index, array) {
		var c1 = action.concat('.enabled');
		var c2 = action.concat('.pin');

		var enabled = self.config.get(c1);
		var pin = self.config.get(c2);

		if(enabled === true){
			self.logger.info('GPIO-Buttons: '+ action + ' on pin ' + pin);
			var j = new Gpio(pin,'in','both');
			j.watch(self.listener.bind(self,action));
			self.triggers.push(j);
		}
	});
		
	return libQ.resolve();
};


GPIOButtons.prototype.clearTriggers = function () {
	var self = this;
	
	self.triggers.forEach(function(trigger, index, array) {
  		self.logger.info("GPIO-Buttons: Destroying trigger " + index);

		trigger.unwatchAll();
		trigger.unexport();		
	});
	
	self.triggers = [];

	return libQ.resolve();	
};


GPIOButtons.prototype.listener = function(action,err,value){
	var self = this;

	var c3 = action.concat('.value');
	var lastvalue = self.config.get(c3);

	// IF change AND high (or low?)
	if(value !== lastvalue && value === 1){
		//do thing
		self[action]();
	}
	// remember value
	self.config.set(c3,value);
};





//Play / Pause
GPIOButtons.prototype.playPause = function() {
  //this.logger.info('GPIO-Buttons: Play/pause button pressed');
  socket.emit('getState','');
  socket.once('pushState', function (state) {
    if(state.status=='play' && state.service=='webradio'){
      socket.emit('stop');
    } else if(state.status=='play'){
      socket.emit('pause');
    } else {
      socket.emit('play');
    }
  });
};

//next on playlist
GPIOButtons.prototype.next = function() {
  //this.logger.info('GPIO-Buttons: next-button pressed');
  socket.emit('next')
};

//previous on playlist
GPIOButtons.prototype.previous = function() {
  //this.logger.info('GPIO-Buttons: previous-button pressed');
  socket.emit('prev')
};

//Volume up
GPIOButtons.prototype.volumeUp = function() {
  //this.logger.info('GPIO-Buttons: Vol+ button pressed');
  socket.emit('volume','+');
};

//Volume down
GPIOButtons.prototype.volumeDown = function() {
  //this.logger.info('GPIO-Buttons: Vol- button pressed\n');
  socket.emit('volume','-');
};

//shutdown
GPIOButtons.prototype.shutdown = function() {
  // this.logger.info('GPIO-Buttons: shutdown button pressed\n');
  this.commandRouter.shutdown();
};

// mute on/off
GPIOButtons.prototype.toggleMute = function() {
  //this.logger.info('GPIO-Buttons: mute button pressed');
  socket.emit('getState','');
  socket.once('pushState', function (state) {
    if(state.mute){
      socket.emit('unmute');
    } else {
      socket.emit('mute');
    }
  });
};

// random on/off
GPIOButtons.prototype.toggleRandom = function() {
  //this.logger.info('GPIO-Buttons: random button pressed');
  socket.emit('getState','');
  socket.once('pushState', function (state) {
    if(state.random){
      socket.emit('setRandom', {"value":false});
    } else {
      socket.emit('setRandom', {"value":true});
    }
  });
};

// repeat all/single/off
GPIOButtons.prototype.toggleRepeat = function() {
  socket.emit('getState','');
  socket.once('pushState', function (state) {
    if(state.repeat){
        if(state.repeatSingle){
            socket.emit('setRepeat', {"value":false, "repeatSingle":false});
        } else {
            socket.emit('setRepeat', {"value":true, "repeatSingle":true});
        }
    } else {
      socket.emit('setRepeat', {"value":true, "repeatSingle":false});
    }
  });
};

// repeat all on/off
GPIOButtons.prototype.toggleRepeatAll = function() {
  socket.emit('getState','');
  socket.once('pushState', function (state) {
    if(state.repeat && !state.repeatSingle){
        socket.emit('setRepeat', {"value":false, "repeatSingle":false});
    } else {
      socket.emit('setRepeat', {"value":true, "repeatSingle":false});
    }
  });
};

// repeat single on/off
GPIOButtons.prototype.toggleRepeatSingle = function() {
  socket.emit('getState','');
  socket.once('pushState', function (state) {
    if(state.repeat && state.repeatSingle){
        socket.emit('setRepeat', {"value":false, "repeatSingle":false});
    } else {
      socket.emit('setRepeat', {"value":true, "repeatSingle":true});
    }
  });
};
